#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_ST7735.h> // Hardware-specific library
#include <SPI.h>
#include <MPU9250_WE.h>
#include <Wire.h>

#define MPU9250_ADDR 0x68
#define AK8963_ADDR 0x0C
#define TFT_CS     9         // TFT CS pin is connected to arduino pin 9
#define TFT_DC     10        // TFT DC pin is connected to arduino pin 10

Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS,  TFT_DC);

MPU9250_WE myMPU9250 = MPU9250_WE(MPU9250_ADDR);

const int numPoints = 128;
float xValues[numPoints] = {0};
float zValues[numPoints] = {0};
int index = 0;
const float scalingFactor = 1000.0; // Increased scaling factor for significant fluctuations

void setup(void) {
  Serial.begin(115200);
  Wire.begin();
  if (!myMPU9250.init()) {
    Serial.println("MPU9250 does not respond");
  } else {
    Serial.println("MPU9250 is connected");
  }

  // Enable I2C bypass mode
  Wire.beginTransmission(MPU9250_ADDR);
  Wire.write(0x37); // INT_PIN_CFG register
  Wire.write(0x02); // Enable bypass mode
  Wire.endTransmission();

  if (!myMPU9250.initMagnetometer()) {
    Serial.println("Magnetometer does not respond");
  } else {
    Serial.println("Magnetometer is connected");
  }

  myMPU9250.setMagOpMode(AK8963_CONT_MODE_100HZ);
  Serial.println("Position your MPU9250 flat and don't move it - calibrating...");
  delay(1000);
  myMPU9250.autoOffsets();
  Serial.println("Done!");
  myMPU9250.setSampleRateDivider(5);
  myMPU9250.setAccRange(MPU9250_ACC_RANGE_2G);
  myMPU9250.enableAccDLPF(true);
  myMPU9250.setAccDLPF(MPU9250_DLPF_6);

  // Initialize the TFT screen
  tft.initR(INITR_BLACKTAB);
  tft.fillScreen(ST7735_BLACK);
  tft.setTextColor(ST7735_WHITE);
  tft.setTextSize(1);
}

void loop() {
  xyzFloat gValue = myMPU9250.getGValues();
  float xValue = gValue.x;
  float zValue = gValue.z;

  // Store the values in the arrays
  xValues[index] = xValue * scalingFactor;
  zValues[index] = (zValue - 1) * scalingFactor;
  
  // Update the index
  index = (index + 1) % numPoints;

  // Clear the graph area
  tft.fillRect(0, 0, 128, 160, ST7735_BLACK);

  // Centering offset
  int centerY = 80; // Center of the screen

  // Draw the graph for x values
  for (int i = 0; i < numPoints - 1; i++) {
    int x1 = i;
    int y1 = centerY - xValues[(index + i) % numPoints];
    int x2 = i + 1;
    int y2 = centerY - xValues[(index + i + 1) % numPoints];
    tft.drawLine(x1, y1, x2, y2, ST7735_RED);
  }

  // Draw the graph for z values
  for (int i = 0; i < numPoints - 1; i++) {
    int x1 = i;
    int y1 = centerY - zValues[(index + i) % numPoints];
    int x2 = i + 1;
    int y2 = centerY - zValues[(index + i + 1) % numPoints];
    tft.drawLine(x1, y1, x2, y2, ST7735_BLUE);
  }

  delay(5); // Update 
}
